/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <app_debug.h>
#include <layoutmgr.h>

#include "define.h"

struct _priv {
	Evas_Object *base;
	Evas_Object *layout;

	layoutmgr *lmgr;
};

static bool _create(layoutmgr *lmgr, void *data)
{
	struct _priv *priv;
	Evas_Object *base, *layout;

	if (!lmgr) {
		_ERR("failed to get layoutmgr");
		return false;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to allocate priv");
		return false;
	}

	base = layoutmgr_get_base(lmgr);
	if (!base) {
		_ERR("failed to get base object");
		free(priv);
		return false;
	}

	layout = elm_layout_add(base);
	if (!layout) {
		_ERR("failed to create layout");
		free(priv);
		return false;
	}

	if (!elm_layout_file_set(layout, EDJEFILE, GRP_GALLERY_LAYOUT)) {
		_ERR("failed to set layout file");
		evas_object_del(layout);
		free(priv);
	}

	priv->base = base;
	priv->layout = layout;
	priv->lmgr = lmgr;

	layoutmgr_set_layout_data(lmgr, LAYOUT_GALLERY, priv);

	return true;
}

static void _destroy(void *layout_data)
{
	struct _priv *priv;

	if (!layout_data) {
		_ERR("failed to get layout data");
		return;
	}

	priv = layout_data;

	evas_object_del(priv->layout);
	free(priv);
}

static void _show(void *layout_data)
{
	struct _priv *priv;

	if (!layout_data) {
		_ERR("failed to layout data");
		return;
	}

	priv = layout_data;

	evas_object_show(priv->layout);
	elm_object_part_content_set(priv->base,
			PART_THUMBNAIL_AREA, priv->layout);
}

static void _hide(void *layout_data)
{
	struct _priv *priv;

	if (!layout_data) {
		_ERR("failed to get layout data");
		return;
	}

	priv = layout_data;

	evas_object_hide(priv->layout);
	elm_object_part_content_unset(priv->base, PART_THUMBNAIL_AREA);
}

static void _update(void *layout_data, int update_type, void *data)
{
}

static layout_class _lclass = {
	.layout_id = LAYOUT_GALLERY,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy,
	.update = _update,
};

layout_class *layout_gallery_get_lclass(void)
{
	return &_lclass;
}
