/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <app_debug.h>
#include <viewmgr.h>
#include <layoutmgr.h>
#include <media_content.h>
#include <media_info.h>
#include <app_media.h>

#include "define.h"
#include "util/controller.h"

#define STYLE_VIEWER_BTN "viewer_btn"
#define PART_VIEWER_BTN "control_btn"

#define VIDEO_COPYRIGHT "Unknown"

enum {
	VIEWER_MOVIE,
	VIEWER_PHOTO,
	VIEWER_VIDEO,
	VIEWER_MAX
};

enum {
	DIR_NONE,
	DIR_PREV,
	DIR_NEXT
};

struct _viewer {
	struct controller *ctl[VIEWER_MAX];
	int cur;
};

struct _playlist {
	Eina_List *list;
	int cur;
};

struct _priv {
	Evas_Object *win;
	Evas_Object *base;
	Evas_Object *photo;
	Evas_Object *photo_pre;
	Evas_Object *favorite;

	struct _viewer viewer;
	struct _playlist playlist;
};

struct _btn_info {
	const char *name;
	int loc;
};

static struct _btn_info btn_movie[] = {
	{
		.name = SRC_BTN_PREV,
		.loc = 0,
	},
	{
		.name = SRC_BTN_REW,
		.loc = 1,
	},
	{
		.name = SRC_BTN_PLAY,
		.loc = 2,
	},
	{
		.name = SRC_BTN_FF,
		.loc = 3,
	},
	{
		.name = SRC_BTN_NEXT,
		.loc = 4,
	},
};

static struct _btn_info btn_photo[] = {
	{
		.name = SRC_BTN_PHOTO_PREV,
		.loc = 0,
	},
	{
		.name = SRC_BTN_PHOTO_NEXT,
		.loc = 4,
	},
};

static struct _btn_info btn_video[] = {
	{
		.name = SRC_BTN_PREV,
		.loc = 0,
	},
	{
		.name = SRC_BTN_REW,
		.loc = 1,
	},
	{
		.name = SRC_BTN_PLAY,
		.loc = 2,
	},
	{
		.name = SRC_BTN_FF,
		.loc = 3,
	},
	{
		.name = SRC_BTN_NEXT,
		.loc = 4,
	},
};

struct _viewer_info {
	struct _btn_info *btns;
	int btn_count;
	int focus_loc;
	void (*callback)(void *, const char *);
};

static void _callback_movie(void *data, const char *ev);
static void _callback_photo(void *data, const char *ev);
static void _callback_video(void *data, const char *ev);

static struct _viewer_info viewer_info[] = {
	{
		.btns = btn_movie,
		.btn_count = 5,
		.focus_loc = 2,
		.callback = _callback_movie,
	},
	{
		.btns = btn_photo,
		.btn_count = 2,
		.focus_loc = 1,
		.callback = _callback_photo,
	},
	{
		.btns = btn_video,
		.btn_count = 5,
		.focus_loc = 4,
		.callback = _callback_video,
	},
};

/* FIXME: test function */
static bool _media(media_info_h media_h, void *dt)
{
	struct _priv *priv;
	app_media *am;

	priv = dt;

	am = app_media_create(media_h);

	priv->playlist.list = eina_list_append(priv->playlist.list, am);

	return true;
}

static void _media_test(struct _priv *priv, int mode)
{
	char *path = "/home/owner/content/Videos/Xmen-Days_of_Future_Past.mp4";
	filter_h filter;
	char buf[1024];
	int r;

	r = media_filter_create(&filter);
	if (r != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("Media Filter Creation Failed");
		return;
	}

	if (mode)
		snprintf(buf, sizeof(buf), "MEDIA_PATH = \"%s\"", path);
	else
		snprintf(buf, sizeof(buf), "MEDIA_TYPE=0 OR MEDIA_TYPE=1");

	r = media_filter_set_condition(filter, buf,
			MEDIA_CONTENT_COLLATE_DEFAULT);
	if (r != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("Fail to set filter condition");
		media_filter_destroy(filter);
		return;
	}

	media_content_connect();

	r = media_info_foreach_media_from_db(filter, _media, priv);
	if (r != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("MEDIA CONTENT ERROR: %d", r);
		media_filter_destroy(filter);
		media_content_disconnect();
		return;
	}

	media_filter_destroy(filter);
	media_content_disconnect();
}
/* FIXME: test function end */

/*
 * NOTE: Workaround
 * we assumed that if video content have the copyright then it's a movie.
 */
static inline bool _check_movie_type(app_media_info *mi)
{
	return strcmp(mi->video->copyright, VIDEO_COPYRIGHT);
}

static void _time_string(char *str, int size, unsigned int ms)
{
	int sec;

	sec = ms / 1000;

	snprintf(str, size, "%02d:%02d:%02d",
			sec / 3600, (sec % 3600) / 60, sec % 60);
}

static void _up_string(char *str)
{
	while (*str) {
		*str = toupper(*str);
		str++;
	}
}

static void _draw_thumbnail(struct _priv *priv, int id, app_media_info *mi)
{
	Evas_Object *obj;

	if (priv->photo_pre) {
		evas_object_del(priv->photo_pre);
		priv->photo_pre = NULL;
	}

	if (priv->photo) {
		evas_object_del(priv->photo);
		priv->photo = NULL;
	}

	obj = elm_image_add(priv->base);
	if (!obj) {
		_ERR("failed to adding image");
		return;
	}

	elm_image_file_set(obj, mi->thumbnail_path, NULL);
	elm_object_part_content_set(priv->base, PART_VIEWER_CONTENT, obj);

	priv->photo = obj;
}

static void _image_loaded(void *data, Evas_Object *obj, void *ev)
{
	struct _priv *priv;

	if (!data || !obj)
		return;

	priv = data;

	if (priv->photo != priv->photo_pre) {
		if (priv->photo)
			evas_object_del(priv->photo);

		priv->photo = obj;
	}

	elm_object_part_content_set(priv->base, PART_VIEWER_CONTENT, obj);
}

static void _image_loaded_detail(void *data, Evas_Object *obj, void *ev)
{
	struct _priv *priv;

	if (!data)
		return;

	priv = data;

	priv->photo_pre = NULL;
}

static void _load_image_file(struct _priv *priv, int id, app_media_info *mi)
{
	Evas_Object *obj;
	int w, h;
	int mode;

	if (id == VIEWER_MOVIE) {
		elm_object_part_content_unset(priv->base, PART_VIEWER_CONTENT);
		return;
	} else if (id == VIEWER_VIDEO) {
		_draw_thumbnail(priv, id, mi);
		return;
	}

	if (!mi->file_path)
		return;

	if (priv->photo_pre) {
		obj = priv->photo_pre;
	} else {
		obj = elm_photocam_add(priv->base);
		if (!obj) {
			_ERR("failed to adding photocam");
			return;
		}

		evas_object_smart_callback_add(obj, "loaded",
				_image_loaded, priv);
		evas_object_smart_callback_add(obj, "loaded,detail",
				_image_loaded_detail, priv);

		elm_object_focus_allow_set(obj, EINA_FALSE);
		elm_scroller_policy_set(obj,
				ELM_SCROLLER_POLICY_AUTO,
				ELM_SCROLLER_POLICY_AUTO);

		/*
		 * set this photocam object to photo_pre (preloaded photo)
		 * On "loaded" callback, photo_pre object will pass to photo
		 * On "loaded,detail" callback, photo_pre object will set to NULL
		 */
		priv->photo_pre = obj;
	}

	elm_photocam_file_set(obj, mi->file_path);

	elm_photocam_image_size_get(obj, &w, &h);
	if (h > w)
		mode = ELM_PHOTOCAM_ZOOM_MODE_AUTO_FIT;
	else
		mode = ELM_PHOTOCAM_ZOOM_MODE_AUTO_FILL;

	elm_photocam_zoom_mode_set(obj, mode);
}

static void _draw_title_bar(struct _priv *priv, int id, app_media_info *mi)
{
	char buf[32] = {0,};
	char day[32] = {0,};
	char date[32] = {0,};
	struct tm tm;

	if (id == VIEWER_MOVIE) {
		elm_object_part_text_set(priv->base,
				PART_VIEWER_TITLE, mi->title);
	} else {
		localtime_r(&mi->modified_time, &tm);

		strftime(date, sizeof(date), "%d %b", &tm);
		strftime(day, sizeof(day), "%a", &tm);
		_up_string(day);

		snprintf(buf, sizeof(buf), "%s, %s", day, date);

		elm_object_part_text_set(priv->base, PART_VIEWER_DATE, buf);

		snprintf(buf, sizeof(buf), "%d / %d", priv->playlist.cur + 1,
				eina_list_count(priv->playlist.list));
		elm_object_part_text_set(priv->base, PART_VIEWER_PAGE, buf);
	}
}

static void _draw_time_info(struct _priv *priv, int id, app_media_info *mi)
{
	char progress[32] = {0,};
	char duration[32] = {0,};
	char total[32] = {0,};

	if (id != VIEWER_PHOTO) {
		_time_string(progress, sizeof(progress), mi->video->position);
		_time_string(duration, sizeof(duration), mi->video->duration);
		snprintf(total, sizeof(total), "/ %s", duration);
	}

	elm_object_part_text_set(priv->base, PART_VIEWER_PROGRESS, progress);
	elm_object_part_text_set(priv->base, PART_VIEWER_TOTAL, total);
}

static void _draw_favorite_icon(struct _priv *priv, int id, app_media_info *mi)
{
	if (!mi->favorite)
		return;

	elm_object_part_content_set(priv->base,
			PART_VIEWER_FAVORITE, priv->favorite);
}

static bool _viewer_show(struct _priv *priv, int cur, int foc)
{
	struct _viewer_info *info;
	struct controller *ctl;
	int id;
	int loc;
	app_media *am;
	app_media_info *mi;

	am = eina_list_nth(priv->playlist.list, cur);
	if (!am) {
		_ERR("failed to get app_media");
		return false;
	}

	priv->playlist.cur = cur;

	mi = app_media_get_info(am);
	if (!mi) {
		_ERR("failed to getting media info");
		return false;
	}

	switch (mi->media_type) {
	case MEDIA_CONTENT_TYPE_IMAGE:
		id = VIEWER_PHOTO;
		break;
	case MEDIA_CONTENT_TYPE_VIDEO:
		if (_check_movie_type(mi))
			id = VIEWER_MOVIE;
		else
			id = VIEWER_VIDEO;
		break;
	default:
		return false;
	}

	info = &viewer_info[id];
	ctl = priv->viewer.ctl[id];
	priv->viewer.cur = id;

	ctl->ops->show(ctl->handle);

	switch (foc) {
	case DIR_PREV:
		loc = 0;
		break;
	case DIR_NEXT:
		loc = info->btn_count - 1;
		break;
	case DIR_NONE:
	default:
		loc = info->focus_loc;
		break;
	}
	ctl->ops->focus(ctl->handle, loc, true);

	_load_image_file(priv, id, mi);

	_draw_title_bar(priv, id, mi);
	_draw_time_info(priv, id, mi);
	_draw_favorite_icon(priv, id, mi);

	return true;
}

static void _viewer_hide(struct _priv *priv)
{
	struct controller *ctl;

	ctl = priv->viewer.ctl[priv->viewer.cur];
	ctl->ops->hide(ctl->handle);

	elm_object_part_content_unset(priv->base, PART_VIEWER_FAVORITE);
}

static bool _viewer_add(struct _priv *priv, int id)
{
	struct _viewer_info *info;
	struct controller *ctl;
	int i;

	info = &viewer_info[id];

	ctl = controller_create(priv->base);
	if (!ctl) {
		_ERR("failed to create controller");
		return false;
	}

	for (i = 0; i < info->btn_count; i++) {
		ctl->ops->add_control(ctl->handle,
				info->btns[i].name, info->btns[i].loc,
				STYLE_VIEWER_BTN, PART_VIEWER_BTN);
	}

	ctl->ops->add_callback(ctl->handle, info->callback, priv);

	priv->viewer.ctl[id] = ctl;

	return true;
}

static void _viewer_delete(struct _priv *priv)
{
	int i;

	for (i = 0; i < VIEWER_MAX; i++)
		controller_destroy(priv->viewer.ctl[i]);
}

static bool _viewer_prev(struct _priv *priv)
{
	int total;
	bool r;

	_viewer_hide(priv);

	total = eina_list_count(priv->playlist.list);

	if (priv->playlist.cur == 0)
		priv->playlist.cur = total - 1;
	else
		priv->playlist.cur--;

	r = _viewer_show(priv, priv->playlist.cur, DIR_PREV);

	return r;
}

static bool _viewer_next(struct _priv *priv)
{
	int total;
	bool r;

	_viewer_hide(priv);

	total = eina_list_count(priv->playlist.list);

	if (priv->playlist.cur == total - 1)
		priv->playlist.cur = 0;
	else
		priv->playlist.cur++;

	r = _viewer_show(priv, priv->playlist.cur, DIR_NEXT);

	return r;
}

static bool _ui_init(struct _priv *priv)
{
	Evas_Object *obj;
	bool r;
	int i;

	for (i = 0; i < VIEWER_MAX; i++) {
		r = _viewer_add(priv, i);
		if (!r)
			goto err;
	}

	obj = elm_image_add(priv->base);
	if (!obj) {
		_ERR("failed to adding image");
		goto err;
	}

	elm_image_file_set(obj, IMAGE_VIEWER_FAVORITE, NULL);
	priv->favorite = obj;

	return true;

err:
	_viewer_delete(priv);
	return false;
}

static void _callback_movie(void *data, const char *ev)
{
}

static void _callback_photo(void *data, const char *ev)
{
	struct _priv *priv;

	if (!data || !ev)
		return;

	priv = data;

	if (!strcmp(ev, SRC_BTN_PHOTO_PREV)) {
		_viewer_prev(priv);
	} else if (!strcmp(ev, SRC_BTN_PHOTO_NEXT)) {
		_viewer_next(priv);
	}
}

static void _callback_video(void *data, const char *ev)
{
	struct _priv *priv;

	if (!data || !ev)
		return;

	priv = data;

	if (!strcmp(ev, SRC_BTN_PREV)) {
		_viewer_prev(priv);
	} else if (!strcmp(ev, SRC_BTN_NEXT)) {
		_viewer_next(priv);
	}
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	Evas_Object *base;
	bool r;

	if (!win) {
		_ERR("failed to get win object");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to allocate priv");
		return NULL;
	}

	base = elm_layout_add(win);
	if (!base) {
		_ERR("failed to create base object");
		free(priv);
		return NULL;
	}

	elm_layout_file_set(base, EDJEFILE, GRP_VIEWER_VIEW);

	evas_object_size_hint_weight_set(base,
			EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, base);

	priv->win = win;
	priv->base = base;

	priv->playlist.list = NULL;
	priv->playlist.cur = 0;

	r = _ui_init(priv);
	if (!r) {
		_ERR("failed to init UI");
		return NULL;
	}

	/* FIXME: test code */
	_media_test(priv, 0);

	viewmgr_set_view_data(VIEW_VIEWER, priv);

	return base;
}

static void _show(void *view_data)
{
	struct _priv *priv;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	/* FIXME: test code */
	_viewer_show(priv, priv->playlist.cur, DIR_NONE);

	evas_object_show(priv->base);
}

static void _hide(void *view_data)
{
	struct _priv *priv;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	_viewer_hide(priv);

	evas_object_hide(priv->base);
}

static void _destroy(void *view_data)
{
	struct _priv *priv;
	app_media *am;

	if (!view_data) {
		_ERR("failed to get view data");
		return;
	}

	priv = view_data;

	_viewer_delete(priv);

	EINA_LIST_FREE(priv->playlist.list, am)
		app_media_destroy(am);

	evas_object_del(priv->base);

	free(priv);
}

static view_class _vclass = {
	.view_id = VIEW_VIEWER,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy,
};

view_class *view_viewer_get_vclass(void)
{
	return &_vclass;
}
